# coding:utf-8

import getpass
import os
from os import path

import paddlehub as hub

_cpu_num = os.system('cat /proc/cpuinfo | grep "processor"| wc -l')
os.system('export CPU_NUM=%d' % (_cpu_num if _cpu_num else 2))

os.system('export FLAGS_eager_delete_tensor_gb=0')
# os.system('export CUDA_VISIBLE_DEVICES=0')

print(':: CPU_NUM: %s' % os.getenv('CPU_NUM'))
print(':: FLAGS_eager_delete_tensor_gb: %s' % os.getenv('FLAGS_eager_delete_tensor_gb'))
print(':: CUDA_VISIBLE_DEVICES: %s' % os.getenv('CUDA_VISIBLE_DEVICES'))

_IN_AI_STUDIO = 'aistudio' == getpass.getuser()
print(':: IN_AI_STUDIO: %s' % _IN_AI_STUDIO)

if _IN_AI_STUDIO:
    os.makedirs(path.join(os.environ['HOME'], '.paddlehub', 'cache'), exist_ok=True)


class PaddleWrap(object):

    def __init__(self, model_name, dataset: hub.dataset.BaseDataset):
        self.in_aistudio = _IN_AI_STUDIO
        self.model_name = model_name
        self.dataset = dataset

        # Load Paddlehub  pretrained model
        self.module = hub.Module(name=model_name)
        self.input_dict, self.output_dict, self.program = self.module.context(trainable=True)

        # Use ImageClassificationReader to read dataset
        data_reader = hub.reader.ImageClassificationReader(
            image_width=self.module.get_expected_image_width(),
            image_height=self.module.get_expected_image_height(),
            images_mean=self.module.get_pretrained_images_mean(),
            images_std=self.module.get_pretrained_images_std(),
            dataset=self.dataset)

        # output_dict["feature_map"]返回了resnet/mobilenet等模型对应的feature_map，可以用于图片的特征表达。
        self.feature_map = self.output_dict["feature_map"]

        # feed_list中的inputs参数指明了resnet/mobilenet等模型的输入tensor的顺序，与ImageClassifierTask返回的结果一致。
        self.feed_list = [self.input_dict["image"].name]

        # learning_rate:        全局学习率。默认为1e-4；
        # optimizer_name:       优化器名称。默认adam；
        # regularization_coeff: 正则化的λ参数。默认为1e-3；
        self.strategy = hub.DefaultFinetuneStrategy(
            learning_rate=1e-4,
            optimizer_name="adam",
            regularization_coeff=1e-3,
        )

        # log_interval:         打印训练日志的周期，默认为10。
        # eval_interval:        进行评估的周期，默认为100。
        # use_pyreader:         是否使用pyreader，默认False。
        # use_data_parallel:    是否使用并行计算，默认False。打开该功能依赖nccl库。
        # save_ckpt_interval:   保存checkpoint的周期，默认为None。
        # use_cuda:             是否使用GPU训练和评估，默认为False。
        # checkpoint_dir:       checkpoint的保存目录，默认为None，此时会在工作目录下根据时间戳生成一个临时目录。
        # num_epoch:            运行的epoch次数，默认为10。
        # batch_size：          每次训练的时候，给模型输入的每批数据大小为32，模型训练时能够并行处理批数据，
        #                       因此batch_size越大，训练的效率越高，但是同时带来了内存的负荷，过大的batch_size可能导致内存不足而无法训练，
        #                       因此选择一个合适的batch_size是很重要的一步；
        # enable_memory_optim:  是否进行内存优化，默认为False。
        # strategy:             finetune的策略。默认为None，此时会使用DefaultFinetuneStrategy策略。
        self.config = hub.RunConfig(
            use_cuda=self.in_aistudio,
            num_epoch=10,
            batch_size=32,
            log_interval=10,
            eval_interval=50,
            checkpoint_dir='../app/ckpt_model',
            strategy=self.strategy)

        # hub.ImageClassifierTask通过输入特征，label与迁移的类别数，可以生成适用于图像分类的迁移任务ImageClassifierTask。
        self.task = hub.ImageClassifierTask(
            data_reader=data_reader,
            feed_list=self.feed_list,
            feature=self.feature_map,
            num_classes=dataset.num_labels,
            config=self.config)



# coding:utf-8

import os

import paddlehub as hub

from paddles.wrap import PaddleWrap

# 直接用PaddleHub提供的数据集
dataset = hub.dataset.Flowers()
paddle_wrap = PaddleWrap('resnet_v2_152_imagenet', dataset)

data = ['../res/classify/daisy/0002.jpg',
        '../res/classify/rose/0001.jpg',
        '../res/classify/test/0002.jpg']

result = paddle_wrap.task.predict(data=data, return_result=True)

for i in range(0, len(data)):
    print('【%s】:\t%s' % (data[i][len('../res/classify/'):], result[i]))



# -*- coding: utf8 -*-

import os

import paddlehub as hub

from paddles.wrap import PaddleWrap

print(":: PWD: %s" % os.getcwd())

# 直接用PaddleHub提供的数据集
dataset = hub.dataset.Flowers()
paddle_wrap = PaddleWrap('resnet_v2_152_imagenet', dataset)

# finetune_and_eval接口来进行模型训练，这个接口在finetune的过程中，会周期性的进行模型效果的评估，以便我们了解整个训练过程的性能变化。
run_states = paddle_wrap.task.finetune_and_eval()



# -*- coding: utf8 -*-

import os
from os import path

import paddlehub as hub

module = hub.Module(name="xception71_imagenet")

img_path = "../res/classify"

# set input dict
input_dict = {
    "image": [path.join(img_path, img) for img in os.listdir(img_path)]
}

# execute predict and print the result
results = module.classification(data=input_dict)
for result in results:
    print('r:', result)

    for item in result:
        print('k: %s: %s' % (item, list(item.keys())))

        for key, values in item.items():
            print('kv: %s=%s' % (key, values))



import os

# print('CPU_NUM: %s' % os.system('cat /proc/cpuinfo | grep "processor"| wc -l'))

cpu_num = os.system('cat /proc/cpuinfo | grep "processor"| wc -l')
print(type(cpu_num))
os.system('export CPU_NUM=%d' % (cpu_num if cpu_num else 3))

print(os.getenv('CPU_NUM'))



# -*- coding: utf8 -*-

import os
import re
from os import path

import paddlehub as hub
import prettytable as pt

img_path = "../res/classify"

# set input dict
input_dict = {
    "image": [path.join(img_path, img) for img in os.listdir(img_path)]
}

model_list = [
    'xception71_imagenet',
    'vgg19_imagenet',
    'shufflenet_v2_imagenet',
    'se_resnext101_32x4d_imagenet',
    'resnext50_vd_64x4d_imagenet',
    'resnext50_vd_32x4d_imagenet',
    'resnext50_64x4d_imagenet',
    'resnext50_32x4d_imagenet',
    'resnext152_vd_64x4d_imagenet',
    'resnext152_64x4d_imagenet',
    'resnet_v2_152_imagenet',
    'pnasnet_imagenet',
    'nasnet_imagenet',
    'mobilenet_v2_imagenet',
    'inception_v4_imagenet',
    'googlenet_imagenet',
    'efficientnetb7_imagenet',
    'dpn131_imagenet',
    'densenet264_imagenet',
    'darknet53_imagenet',
    'alexnet_imagenet'
]

field_list = ['MODEL/CLASSIFY']
tb_list = []

max_dict = {}

for model_name in model_list:
    # 创建每行数据格式
    row_array = ['0'] * len(field_list)

    # 首列：模型名称
    row_array[0] = model_name[:model_name.rindex('_')]

    module = hub.Module(name=model_name)
    results = module.classification(data=input_dict)
    # [[{,},],]
    for result in results:
        # [{,},]
        for item in result:
            # {}, 查找最大的匹配率的对象
            k_list = list(item.keys())
            v_list = list(item.values())

            max_value = max(v_list)

            key = k_list[v_list.index(max_value)]
            value = str(max_value)

            if key not in max_dict or max_dict[key] < max_value:
                max_dict[key] = max_value

            # 填充行数据
            index = field_list.index(key) if key in field_list else -1
            if index >= 0:
                row_array[index] = value

            elif max_value >= 0.4:
                # 新添加key，value
                field_list.append(key)
                row_array.append(value)

                # 调整历史数据，从第3行开始填充历史空数据
                if len(tb_list) > 0:
                    for list_raw in tb_list:
                        list_raw.append('0')

    # 添加一行数据
    tb_list.append(row_array)

# pattern = re.compile(r'^[-+]?[-0-9]\d*\.\d*|[-+]?\.?[0-9]\d*$')
pattern = re.compile(r'^[0-9]+\.[0-9]*$')
for column in range(1, len(field_list)):
    max_value = max_dict.get(field_list[column])

    for row in range(0, len(tb_list)):
        value = float(tb_list[row][column])

        # tb_list[row][column] = '%.4f ' % value if value else ' '
        format_value = '*%.6f ' % value if value == max_value else (' %.6f ' % value if value else '')
        tb_list[row][column] = format_value[:7]

# 按行添加数据
tb = pt.PrettyTable()

tb.field_names = field_list
tb._rows = tb_list

tb.align[field_list[0]] = "l"

print(tb)



# -*- coding: UTF-8 -*-

import re
from os import path

from setuptools import setup, find_packages


def get_property(prop, project):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(path.join(project, '__init__.py')).read())

    return result.group(1)


with open('README.md', encoding='utf-8') as readme_file:
    _long_description = readme_file.read()

setup(
    author='handy',
    author_email='cvdnn@foxmail.com',
    url='https://pypi.org/project/paddle-wrap/',

    license='Apache v2.0',
    description='paddle wrap',
    long_description=_long_description,
    long_description_content_type='text/markdown',
    keywords=["paddle", "paddlehub"],

    name='paddle-wrap',
    version='0.0.4',

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['paddlehub'],

    # 此项需要，否则卸载时报windows error
    zip_safe=False
)



# -*- coding: utf8 -*-

import paddlehub as hub
import cv2
import os

from os import path

module = hub.Module(name="pyramidbox_lite_mobile_mask")
# module = hub.Module(name="pyramidbox_lite_server_mask")

img_path = '../res/mask'

# set input dict
# input_dict = {
#     "data": [cv2.imread(test_img_path)]
# }
#
# results = module.face_detection(data=input_dict)

# or
input_dict = {
    "image": [path.join(img_path, img) for img in os.listdir(img_path)]
}
"""
data：dict类型，支持data和image两种key，其中：
    key为data，表示value为待检测的图片数据，numpy.array类型，shape为[H, W, C]，BGR格式。
    key为image，表示value为待检测的图片路径。
use_gpu：bool类型，表示是否使用GPU进行预测，需要 配合环境变量 CUDA_VISIBLE_DEVICES使用
batch_size：int类型，表示批量预测时的大小，默认为1，设置越大能够同时处理越多图片，但是会带来更高的显存消耗
shrink：float类型 (0, 1]，用于设置图片的缩放比例，该值越大，则对于输入图片中的小尺寸人脸有更好的检测效果（模型计算成本越高），反之则对于大尺寸人脸有更好的检测效果。
use_multi_scale：bool类型，用于设置是否开启多尺度的人脸检测，开启多尺度人脸检测能够更好的检测到输入图像中不同尺寸的人脸，但是会增加模型计算量，降低预测速度
"""
results = module.face_detection(data=input_dict,
                                use_gpu=False,
                                batch_size=1,
                                shrink=0.5,
                                use_multi_scale=False)
for result in results:
    print(result)
